import java.util.ArrayList;
import java.util.Random;

public class GeometricBrownianMotionDemo {

    private static Random rand = new Random();

    public static void main(String[] args) {        

        double s0 = 100; // spot price
        double r = 0.05; // annual risk-free interest rate
        double v = 0.25; // annual volatility
        double T = 1; // time to maturity (yr)
        int periods = 1000; // samples

        StdDraw.setCanvasSize(600, 600);
        StdDraw.setXscale(0, periods);
        StdDraw.setYscale(0, 250);
        StdDraw.line(0, 0, periods, 0);
        StdDraw.line(0, 0, 0, 250);
        
        
        for (int j = 1; j <= 100; j++) {

            StdDraw.setPenColor(StdDraw.RED);

            double[] prices = GBM(s0, r, v, T, periods);
            for (int i = 0; i < prices.length; i++)
                StdDraw.filledCircle(i, prices[i], 0.9);

            /*
            ArrayList<Double> prices = GBM(s0, r, v, T, periods);
            for (int i = 0; i < prices.size(); i++)
                StdDraw.filledCircle(i, prices.get(i), 0.9);
            */
        }

    }

    // native array
    public static double[] GBM(double s0, double r, double v, double T, int N) {

        double dt = T / N, sqdt = Math.sqrt(dt);

        double[] prices = new double[N + 1];
        prices[0] = s0;
        for (int i = 1; i <= N; i++) {
            prices[i] = prices[i - 1] * Math.exp((r - 0.5 * v * v) * dt + v * sqdt * rand.nextGaussian());
        }
        return prices;

    }

    /*
    // array list
    public static ArrayList<Double> GBM(double s0, double r, double v, double T, int N) {

        double dt = T / N, sqdt = Math.sqrt(dt);

        ArrayList<Double> prices = new ArrayList<>();
        prices.add(s0);
        for (int i = 1; i <= N; i++) {
            double s = prices.get(i - 1) * Math.exp((r - 0.5 * v * v) * dt + v * sqdt * rand.nextGaussian());
            prices.add(s);
        }
        return prices;

    }
    */

    /*
    // Stream API: functional programming
    public static ArrayList<Double> GBM(double s0, double r, double v, double T, int N) {

        double dt = T / N, sqdt = Math.sqrt(dt);
        // todo: how to do cumprod by Stream API?
        Stream.iterate(0, i -> i + 1).limit(N + 1).reduce(s0, (s, x) -> s * Math.exp((r - 0.5 * v * v) * dt + v * sqdt * rand.nextGaussian()))

    }
    */

}
